(* File: search.ml *)
(* Author: Chad E Brown *)
(* Created: October 2010 *)

open Flags
open State
open Syntax
open Minisatinterface

(*** Compute minimal unsatisfiable core with picomus - This should be moved somewhere more reasonable. - Chad, Nov 9, 2010 ***)
let minimal_unsatisfiable_core clauses =
  if (!verbosity > 0) then Printf.printf "About to call picomus\n";
  let (myout,myin,myerr) = Unix.open_process_full ((!Config.picomus) ^ " - -") [| |] in
  Printf.fprintf myin "p cnf %d %d\n" (max_atom ()) (List.length clauses);
  if (!verbosity > 0) then Printf.printf "About to send clauses to picomus\n";
  List.iter
    (fun c ->
      Printf.fprintf myin "%s 0\n" (String.concat " " (List.map string_of_int c))
    )
    clauses;
  close_out myin;
  if (!verbosity > 0) then Printf.printf "About to read result from picomus\n";
  match (input_line myout) with
  | "s UNSATISFIABLE" ->
      if (!verbosity > 0) then Printf.printf "picomus 1\n";
      let ret = ref [] in
      let cl = ref [] in
      let neg = ref false in
      let atm = ref 0 in
      begin
	if (!verbosity > 0) then Printf.printf "picomus 2\n";
	ignore (input_line myout);
	if (!verbosity > 0) then Printf.printf "picomus 3\n";
	try
	  (*** This is not the best way to do this.  I'll read in one character at a time, building lit ints, clauses (cl), and pushing clauses onto ret to return ***)
	  while true do
	    if (!verbosity > 0) then Printf.printf "picomus 4\n";
	    let c = input_byte myout in (*** will throw End_of_file when finished ***)
	    if (!verbosity > 0) then Printf.printf "picomus 5 %d\n" c;
	    if (c = 48) then
	      begin
		if ((!atm) = 0) then (** end of clause **)
		  begin
		    ret := (List.rev (!cl))::!ret;
		    cl := []
		  end
		else
		  atm := 10 * (!atm)
	      end
	    else if ((c > 48) && (c <= 57)) then
	      atm := 10 * (!atm) + (c - 48)
	    else if (c = 45) then (*** negative literal ***)
	      neg := true
	    else (*** otherwise assume this is a blank space ***)
	      begin
		if ((!atm) > 0) then
		  begin
		    if (!neg) then
		      cl := ((-(!atm))::(!cl))
		    else
		      cl := ((!atm)::(!cl));
		    neg := false;
		    atm := 0;
		  end
	      end
	  done;
	  raise End_of_file (*** Unreachable ***)
	with
	| End_of_file -> (!ret)
      end
  | _ -> raise (GenericError "strange return from picomus") (*** In case this happens, maybe I should just return all the clauses ***)

let rec new_search_clause c r =
  List.iter (fun l -> minisat_addLit l) c;
  minisat_addClause();
  insert_search_clause c r;
  match minisat_search() with
  | 20 -> (*** UNSATISFIABLE ***)
	  if (!mkproofterm) then
	    let cri = !clause_ruleinfo in (*** remember the clauses and this hashtable for them ***)
(*** To test with minimal unsatisfiable core: ***)
	    let minclauses = minimal_unsatisfiable_core (!clauses) in
(*** To test all clauses: ***)
(***	    let minclauses = !clauses in ***)
	    raise (Unsatisfiable (Some (SearchR (minclauses,Hashtbl.find cri))))
	  else
	    raise (Unsatisfiable None)
  | _ -> ()


let fake_delay = ref 0;;

let fake_search_main () =
  try
    while true do
      let op = getNext() in
      if (!verbosity > 0) then Printf.printf "Option: %s\n" (searchoption_str op);
      match op with
      | ProcessProp1(m) ->
	  begin
	    if (not (Hashtbl.mem processed m)) then
	      begin
		Hashtbl.add processed m ();
		let mlit = get_literal m in
		Printf.printf "working on %d %s\n" mlit (trm_str m);
		match m with
		| Ap(Ap(Imp,Ap(Forall(a),m1)),False) ->
		    begin
		      let (ws,w) = get_fresh_name a in
		      print_string("Using Fresh Witness " ^ (trm_str w)); print_newline();
		      let m1w = (norm name_def (Ap(m1,w))) in
		      print_string(" m1w = " ^ (trm_str m1w)); print_newline();
		      let nm1w = (normneg m1w) in
		      insertWithPriority (get_int_flag "EXISTS_DELAY") (ProcessProp1 nm1w);
		      let nm1wlit = get_literal nm1w in
		      new_search_clause [-mlit;nm1wlit]
			(if (!mkproofterm) then (Some(FreshRule(a,m1,ws))) else None)
		    end
		| Ap(Ap(Imp,nm),False) ->
		    let r = if (!mkproofterm) then (Some(NegPropRule(nm))) else None in
		    let (h,args) = head_spine nm in
		    begin
		      match h with
		      | False -> () (*** do nothing ***)
		      | Imp ->
			  begin
			    match args with
			      [m1;m2] ->
				begin
				  insertWithPriority (get_int_flag "POST_NOR_L_DELAY") (ProcessProp1 m1);
				  insertWithPriority (get_int_flag "POST_NOR_R_DELAY") (ProcessProp1 (normneg m2));
				  let m1lit = get_literal m1 in
				  let nm2lit = get_literal (normneg m2) in
				  new_search_clause [-mlit;m1lit] r;
				  new_search_clause [-mlit;nm2lit] r
				end
			    | _ -> raise (GenericError "foo3")
			  end
		      | Forall(_) -> raise (GenericError "Neg Forall should have had exactly one argument; there's a bug.")
		      | Eq(_) ->
			  begin
			    match args with
			      [m1;m2] ->
				begin
				  (** for now, just handle reflexivity **)
				  if (m1 = m2) then new_search_clause [-mlit] r;
				  (*** to do [decompose, confront, FE, BE] ***)
				end
			    | _ -> raise (GenericError "foo3")
			  end
		      | Choice(_) -> () (** choice, accessible **)
		      | Name(x,_) -> (** negative atom **)
			  let pmd = get_int_flag "PRE_MATING_DELAY_NEG" in
			  Hashtbl.add natoms x (mlit,args);
			  List.iter (fun (plit,pargs) ->
			    if (not (pargs = args)) then
			      insertWithPriority pmd (Mating(plit,mlit,pargs,args))
				    )
			    (Hashtbl.find_all patoms x)
		      | _ -> raise (GenericError ("Unhandled case (were logical constants normalized?) h:" ^ (trm_str h)))
		    end
		| Ap(Forall(a),m1) -> (*** Instantiate with every name of the type ***)
		    begin
		      Hashtbl.iter
			(fun _ (w,b) ->
			  if (a = b) then
			    begin
			      print_string("Instantiating with " ^ (trm_str w)); print_newline();
			      let m1w = (norm name_def (Ap(m1,w))) in
			      insertWithPriority (get_int_flag "FORALL_DELAY") (ProcessProp1 m1w);
			      if (!fake_delay < 10) then (** process forall's more than once to get later instantiations, but stop to terminate and call minisat **)
				begin
				  incr fake_delay;
				  Hashtbl.remove processed m;
				  insertWithPriority (100 * (!fake_delay)) (ProcessProp1(m))
				end;
			      let m1wlit = get_literal m1w in
			      new_search_clause [-mlit;m1wlit]
				(if (!mkproofterm) then (Some(InstRule(a,m1,w))) else None)
			    end)
			name_trm
		    end
		| _ ->
		    let r = if (!mkproofterm) then (Some(PosPropRule(m))) else None in
		    begin
		      let (h,args) = head_spine m in
		      match h with
		      | False ->
			  let flit = get_literal False in
			  new_search_clause [-flit] r
		      | Imp ->
			  begin
			    match args with
			      [m1;m2] ->
				begin
				  insertWithPriority (get_int_flag "POST_NOR_L_DELAY") (ProcessProp1 (normneg m1));
				  insertWithPriority (get_int_flag "POST_NOR_R_DELAY") (ProcessProp1 m2);
				  let nm1lit = get_literal (normneg m1) in
				  let m2lit = get_literal m2 in
				  new_search_clause [(-mlit);nm1lit;m2lit] r
				end
			    | [] -> raise (GenericError "foo")
			  end
		      | Forall(_) -> raise (GenericError "Forall should have had exactly one argument; there's a bug.")
		      | Eq(_) -> () (*** to do [confront, FQ, BQ] ***)
		      | Choice(_) -> () (** choice, accessible **)
		      | Name(x,_) -> (** positive atom **)
			  let pmd = get_int_flag "PRE_MATING_DELAY_POS" in
			  Hashtbl.add patoms x (mlit,args);
			  List.iter (fun (nlit,nargs) ->
			    if (not (args = nargs)) then
			      insertWithPriority pmd (Mating(mlit,nlit,args,nargs));
				    )
			    (Hashtbl.find_all natoms x)
		      | _ -> raise (GenericError ("Unhandled case (were logical constants normalized?) h:" ^ (trm_str h)))
		    end
	      end; (*** ; because there's no else ***)
	  end
      | Mating(plit,nlit,pargs,nargs) ->
	  let pmd = get_int_flag "POST_MATING_DELAY" in
	  let dlits = List.map2
	      (fun pa na ->
		let deq = normneg (ueq pa na) in
		insertWithPriority pmd (ProcessProp1 deq);
		get_literal deq
	      )
	      pargs nargs
	  in
	  new_search_clause
	    ((-plit)::(-nlit)::dlits)
	    (if (!mkproofterm) then (Some (MatingRule(plit,nlit))) else None)
      | _ -> raise (GenericError "foo")
    done
  with Not_found -> raise Satisfiable
      

let fake_search_1 () =
  begin
    print_string "This is a fake search (1)";
    print_newline();
    List.iter (fun m ->
      insertWithPriority 0 (ProcessProp1 m);
      let mlit = get_literal m in
      insert_assumption_clause [mlit]
	)
      (!initial_branch);
    fake_search_main ()
  end

let rec fake_search_pre p b sgn =
  match p with
    (m::pr) ->
      begin
	match m with
	| Ap(Ap(Imp,Ap(Ap(Eq(_),m1),m2)),False) when (m1 = m2) ->
	    if (!mkproofterm) then
	      raise (Unsatisfiable (Some NegReflR))
	    else
	      raise (Unsatisfiable None)
	| Ap(Ap(Imp,Ap(Forall(a),m1)),False) ->
	    begin
	      let (ws,w) = get_fresh_name a in
	      print_string("Using Fresh Witness " ^ (trm_str w)); print_newline();
	      let m1w = (norm name_def (Ap(m1,w))) in
	      print_string(" m1w = " ^ (trm_str m1w)); print_newline();
	      let nm1w = (normneg m1w) in
	      begin
		try
		  fake_search_pre (nm1w::pr) b sgn
		with
		| Unsatisfiable(Some(r1)) ->
		    raise (Unsatisfiable(Some(NegAllR(a,m1,ws,r1))))
	      end
	    end
	| Ap(Ap(Imp,nm),False) ->
	    let (h,args) = head_spine nm in
	    begin
	      match h with
	      | False -> fake_search_pre pr b sgn (*** drop ~False ***)
	      | Imp ->
		  begin
		    match args with
		      [m1;m2] ->
			begin
			  try
			    fake_search_pre (m1::(normneg m2)::pr) b sgn
			  with
			  | Unsatisfiable(Some(r1)) ->
			      raise (Unsatisfiable (Some (NegImpR(m1,m2,r1))))
			end
		    | _ -> raise (GenericError "foo3")
		  end
	      | Eq(Ar(a1,a2)) ->
		  begin
		    match args with
		      [m1;m2] ->
			let m1s = shift m1 0 1 in
			let m2s = shift m2 0 1 in
			begin
			  try
			    fake_search_pre
			      ((norm name_def (normneg (forall a1 (eq a2 (Ap(m1s,DB(0,a1))) (Ap(m2s,DB(0,a1)))))))::pr)
			      b sgn
			  with
			  | Unsatisfiable(Some(r1)) ->
			      raise (Unsatisfiable (Some (NegEqFR(a1,a2,m1,m2,r1))))
			end
		    | _ -> raise (GenericError "foo3")
		  end		  
	      | Eq(Prop) ->
		  begin
		    match args with
		      [m1;m2] ->
			begin
			  try
			    fake_search_pre (m1::(normneg m2)::pr) b sgn
			  with
			  | Unsatisfiable(Some(r1)) ->
			      begin
				try
				  reset_search (); incr sgn;
				  fake_search_pre ((normneg m1)::m2::pr) b sgn
				with
				| Unsatisfiable(Some(r2)) ->
				    raise (Unsatisfiable (Some (NegEqOR(m1,m2,r1,r2))))
			      end
			  | Unsatisfiable(None) ->
			      reset_search (); incr sgn;
			      fake_search_pre ((normneg m1)::m2::pr) b sgn
			end
		    | _ -> raise (GenericError "foo3")
		  end
	      | _ ->
		  fake_search_pre pr (m::b) sgn
	    end
	| _ ->
	    let (h,args) = head_spine m in
	    begin
	      match h with
	      | False -> if (!mkproofterm) then raise (Unsatisfiable (Some FalseR)) else raise (Unsatisfiable None)
	      | Imp ->
		  begin
		    match args with
		      [m1;m2] ->
			begin
			  try
			    fake_search_pre ((normneg m1)::pr) b sgn
			  with
			  | Unsatisfiable(Some(r1)) ->
			      begin
				try
				  reset_search (); incr sgn;
				  fake_search_pre (m2::pr) b sgn
				with
				| Unsatisfiable(Some(r2)) ->
				    raise (Unsatisfiable (Some (ImpR(m1,m2,r1,r2))))
			      end
			  | Unsatisfiable(None) ->
			      reset_search (); incr sgn;
			      fake_search_pre (m2::pr) b sgn
			end
		    | _ -> raise (GenericError "foo3")
		  end
	      | Eq(Prop) ->
		  begin
		    match args with
		      [m1;m2] ->
			begin
			  try
			    fake_search_pre (m1::m2::pr) b sgn
			  with
			  | Unsatisfiable(Some(r1)) ->
			      begin
				try
				  reset_search (); incr sgn;
				  fake_search_pre ((normneg m1)::(normneg m2)::pr) b sgn
				with
				| Unsatisfiable(Some(r2)) ->
				    raise (Unsatisfiable (Some (EqOR(m1,m2,r1,r2))))
			      end
			  | Unsatisfiable(None) ->
			      reset_search (); incr sgn;
			      fake_search_pre ((normneg m1)::(normneg m2)::pr) b sgn
			end
		    | _ -> raise (GenericError "foo3")
		  end
	      | _ ->
		  fake_search_pre pr (m::b) sgn
	    end
      end
  | [] ->
      begin
	Printf.printf "Searching on Subgoal %d\n" (!sgn);
	List.iter (fun m ->
	  insertWithPriority 0 (ProcessProp1 m);
	  let mlit = get_literal m in
	  insert_assumption_clause [mlit]
		  )
	  b;
	fake_search_main ();
      end
	
let fake_search_0 () =
  begin
    print_string "Search code is not yet written 0";
    print_newline();
    fake_search_pre (!initial_branch) [] (ref 1)
  end

let search () =
(***    print_string "Search code is not yet written"; print_newline(); ***)
  begin (*** This function is not yet written.  The current version is for testing.  I'll use the flag EXISTS_DELAY just to decide what to return. ***)
    let _ = minisat_init() in
    let w = get_int_flag "EXISTS_DELAY" in
    if (w = 0) then (* e.g., -m mode2 *)
      fake_search_0 ()
    else if (w = 1) then (* e.g., -m mode1 *)
      fake_search_1 ()
    else if (w = 2) then (* e.g., -m mode5 *)
      begin
	print_string "Search code is not yet written 2";
	print_newline();
      end
    else
      raise Satisfiable (*** just for testing ***)
  end

